@@ -1,5 +1,5 @@
-/* $Id: sycls.c,v 1.72 2006/07/13 06:35:07 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* source: sycls.c */
+/* Copyright Gerhard Rieger 2001-2008 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* explicit system call and C library trace function, for those who miss strace
@@ -374,6 +374,9 @@ static void prtstat(const char *func, struct stat *buf, int result) {
 static void prtstat64(const char *func, struct stat64 *buf, int result) {
    char txt[256], *t = txt;
 
+   if (result < 0) {
+      sprintf(t, "%s(, {}) -> %d", func, result);
+   } else {
    t += sprintf(t, "%s(, {"F_st_dev","F_st64_ino","F_mode","F_st_nlink","F_uid","F_gid,
 		func, buf->st_dev, buf->st_ino,
 		buf->st_mode, buf->st_nlink, buf->st_uid, buf->st_gid);
@@ -388,6 +391,7 @@ static void prtstat64(const char *func, struct stat64 *buf, int result) {
    t += sprintf(t, ","F_st64_blocks, buf->st_blocks);
 #endif
    sprintf(t, ",...}) -> %d", result);
+   }
    Debug(txt);
 }
 #endif /* defined(HAVE_STAT64) || defined(HAVE_FSTAT64) || defined(HAVE_LSTAT64) */
@@ -589,6 +593,16 @@ int Ioctl(int d, int request, void *argp) {
    return retval;
 }
 
+int Ioctl_int(int d, int request, int arg) {
+   int retval, _errno;
+   Debug3("ioctl(%d, 0x%x, %d)", d, request, arg);
+   retval = ioctl(d, request, arg);
+   _errno = errno;
+   Debug1("ioctl() -> %d", retval);
+   errno = _errno;
+   return retval;
+}
+
 int Close(int fd) {
    int retval, _errno;
    Info1("close(%d)", fd);
@@ -673,9 +687,21 @@ int Chmod(const char *path, mode_t mode) {
 /* we only show the first struct pollfd; hope this is enough for most cases. */
 int Poll(struct pollfd *ufds, unsigned int nfds, int timeout) {
    int result;
-   Debug4("poll({%d, 0x%02hx, }, %u, %d)", ufds[0].fd, ufds[0].events, nfds, timeout);
+   if (nfds == 4) {
+      Debug10("poll({%d,0x%02hx,}{%d,0x%02hx,}{%d,0x%02hx,}{%d,0x%02hx,}, %u, %d)",
+	      ufds[0].fd, ufds[0].events, ufds[1].fd, ufds[1].events,
+	      ufds[2].fd, ufds[2].events, ufds[3].fd, ufds[3].events,
+	      nfds, timeout);
+   } else {
+      Debug4("poll({%d,0x%02hx,}, , %u, %d)", ufds[0].fd, ufds[0].events, nfds, timeout);
+   }
    result = poll(ufds, nfds, timeout);
-   Debug2("poll(, {,, 0x%02hx}) -> %d", ufds[0].revents, result);
+   if (nfds == 4) {
+      Debug5("poll(, {,,0x%02hx}{,,0x%02hx}{,,0x%02hx}{,,0x%02hx}) -> %d",
+	     ufds[0].revents, ufds[1].revents, ufds[2].revents, ufds[3].revents, result);
+   } else {
+      Debug2("poll(, {,,0x%02hx}) -> %d", ufds[0].revents, result);
+   }
    return result;
 }
 #endif /* HAVE_POLL */
@@ -760,6 +786,14 @@ int Sigaction(int signum, const struct sigaction *act,
 }
 #endif /* HAVE_SIGACTION */
 
+int Sigprocmask(int how, const sigset_t *set, sigset_t *oset) {
+   int retval;
+   Debug3("sigprocmask(%d, %p, %p)", how, set, oset);
+   retval = sigprocmask(how, set, oset);
+   Debug1("sigprocmask() -> %d", retval);
+   return retval;
+}
+
 unsigned int Alarm(unsigned int seconds) {
    unsigned int retval;
    Debug1("alarm(%u)", seconds);
@@ -830,7 +864,7 @@ int Socketpair(int d, int type, int protocol, int sv[2]) {
    return result;
 }
 
-#if WITH_SOCKET
+#if _WITH_SOCKET
 int Socket(int domain, int type, int protocol) {
    int result, _errno;
    Debug3("socket(%d, %d, %d)", domain, type, protocol);
@@ -840,14 +874,14 @@ int Socket(int domain, int type, int protocol) {
    errno = _errno;
    return result;
 }
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
 
-#if WITH_SOCKET
+#if _WITH_SOCKET
 int Bind(int sockfd, struct sockaddr *my_addr, int addrlen) {
    int result, _errno;
    char infobuff[256];
 
-   sockaddr_info(my_addr, infobuff, sizeof(infobuff));
+   sockaddr_info(my_addr, addrlen, infobuff, sizeof(infobuff));
    Debug3("bind(%d, %s, "F_Zd")", sockfd, infobuff, addrlen);
    result = bind(sockfd, my_addr, addrlen);
    _errno = errno;
@@ -855,9 +889,9 @@ int Bind(int sockfd, struct sockaddr *my_addr, int addrlen) {
    errno = _errno;
    return result;
 }
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
 
-#if WITH_SOCKET
+#if _WITH_SOCKET
 int Connect(int sockfd, const struct sockaddr *serv_addr, int addrlen) {
    int result, _errno;
    char infobuff[256];
@@ -879,7 +913,7 @@ int Connect(int sockfd, const struct sockaddr *serv_addr, int addrlen) {
 #else
    Debug4("connect(%d, {%d,%s}, "F_Zd")",
 	  sockfd, serv_addr->sa_family,
-	  sockaddr_info(serv_addr, infobuff, sizeof(infobuff)),
+	  sockaddr_info(serv_addr, addrlen, infobuff, sizeof(infobuff)),
 	  addrlen);
 #endif
    result = connect(sockfd, serv_addr, addrlen);
@@ -888,9 +922,9 @@ int Connect(int sockfd, const struct sockaddr *serv_addr, int addrlen) {
    errno = _errno;
    return result;
 }
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
 
-#if WITH_SOCKET
+#if _WITH_SOCKET
 int Listen(int s, int backlog) {
    int result, _errno;
    Debug2("listen(%d, %d)", s, backlog);
@@ -900,9 +934,9 @@ int Listen(int s, int backlog) {
    errno = _errno;
    return result;
 }
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
 
-#if WITH_SOCKET
+#if _WITH_SOCKET
 /* don't forget to handle EINTR when using Accept() ! */
 int Accept(int s, struct sockaddr *addr, socklen_t *addrlen) {
    int result, _errno;
@@ -912,19 +946,20 @@ int Accept(int s, struct sockaddr *addr, socklen_t *addrlen) {
    _errno = errno;
    if (result >= 0) {
       char infobuff[256];
-      sockaddr_info(addr, infobuff, sizeof(infobuff));
+      sockaddr_info(addr, *addrlen, infobuff, sizeof(infobuff));
       Info5("accept(%d, {%d, %s}, "F_Zd") -> %d", s,
-	     addr->sa_family, sockaddr_info(addr, infobuff, sizeof(infobuff)),
-	     *addrlen, result);
+	    addr->sa_family,
+	    sockaddr_info(addr, *addrlen, infobuff, sizeof(infobuff)),
+	    *addrlen, result);
    } else {
       Debug1("accept(,,) -> %d", result);
    }
    errno = _errno;
    return result;
 }
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
 
-#if WITH_SOCKET
+#if _WITH_SOCKET
 int Getsockname(int s, struct sockaddr *name, socklen_t *namelen) {
    int result, _errno;
    char infobuff[256];
@@ -935,13 +970,14 @@ int Getsockname(int s, struct sockaddr *name, socklen_t *namelen) {
    /*Debug2("getsockname(,, {"F_socklen"}) -> %d",
       *namelen, result);*/
    Debug3("getsockname(, {%s}, {"F_socklen"}) -> %d",
-	  sockaddr_info(name, infobuff, sizeof(infobuff)), *namelen, result);
+	  sockaddr_info(name, *namelen, infobuff, sizeof(infobuff)),
+	  *namelen, result);
    errno = _errno;
    return result;
 }
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
 
-#if WITH_SOCKET
+#if _WITH_SOCKET
 int Getpeername(int s, struct sockaddr *name, socklen_t *namelen) {
    int result, _errno;
    char infobuff[256];
@@ -949,15 +985,15 @@ int Getpeername(int s, struct sockaddr *name, socklen_t *namelen) {
    Debug4("getpeername(%d, %p, %p{"F_socklen"})", s, name, namelen, *namelen);
    result = getpeername(s, name, namelen);
    _errno = errno;
-   sockaddr_info(name, infobuff, sizeof(infobuff));
+   sockaddr_info(name, *namelen, infobuff, sizeof(infobuff));
    Debug3("getpeername(, {%s}, {"F_socklen"}) -> %d",
 	  infobuff, *namelen, result);
    errno = _errno;
    return result;
 }
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
 
-#if WITH_SOCKET
+#if _WITH_SOCKET
 int Getsockopt(int s, int level, int optname, void *optval, socklen_t *optlen) {
    int result, _errno;
    Debug5("getsockopt(%d, %d, %d, %p, {"F_Zd"})",
@@ -969,12 +1005,12 @@ int Getsockopt(int s, int level, int optname, void *optval, socklen_t *optlen) {
    errno = _errno;
    return result;
 }
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
 
-#if WITH_SOCKET
+#if _WITH_SOCKET
 int Setsockopt(int s, int level, int optname, const void *optval, int optlen) {
    int result, _errno;
-   if (optlen == sizeof(int)) {
+   if (optlen <= sizeof(int)) {
       Debug5("setsockopt(%d, %d, %d, {0x%x}, %d)",
        s, level, optname, *(unsigned int *)optval, optlen);
    } else {
@@ -985,14 +1021,13 @@ int Setsockopt(int s, int level, int optname, const void *optval, int optlen) {
    }
    result = setsockopt(s, level, optname, optval, optlen);
    _errno = errno;
-   Debug3("setsockopt() -> (,,, 0x%08x, %d), %d",
-       *(unsigned int *)optval, optlen, result);
+   Debug1("setsockopt() -> %d", result);
    errno = _errno;
    return result;
 }
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
 
-#if WITH_SOCKET
+#if _WITH_SOCKET
 int Recv(int s, void *buf, size_t len, int flags) {
    int retval, _errno;
    Debug4("recv(%d, %p, "F_Zu", %d)", s, buf, len, flags);
@@ -1002,9 +1037,9 @@ int Recv(int s, void *buf, size_t len, int flags) {
    errno = _errno;
    return retval;
 }
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
 
-#if WITH_SOCKET
+#if _WITH_SOCKET
 int Recvfrom(int s, void *buf, size_t len, int flags, struct sockaddr *from,
 	     socklen_t *fromlen) {
    int retval, _errno;
@@ -1015,32 +1050,49 @@ int Recvfrom(int s, void *buf, size_t len, int flags, struct sockaddr *from,
    _errno = errno;
    if (from) {
       Debug4("recvfrom(,,,, {%d,%s}, "F_Zd") -> %d",
-	  from->sa_family, sockaddr_info(from, infobuff, sizeof(infobuff)),
-	  *fromlen, retval);
+	     from->sa_family,
+	     sockaddr_info(from, *fromlen, infobuff, sizeof(infobuff)),
+	     *fromlen, retval);
    } else {
       Debug1("recvfrom(,,,, NULL, NULL) -> %d", retval);
    }
    errno = _errno;
    return retval;
 }
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
 
-#if WITH_SOCKET
+#if _WITH_SOCKET
 int Recvmsg(int s, struct msghdr *msgh, int flags) {
    int retval, _errno;
    char infobuff[256];
-   Debug3("recvmsg(%d, %p, %d)", s, msgh, flags);
+#if defined(HAVE_STRUCT_MSGHDR_MSGCONTROL) && defined(HAVE_STRUCT_MSGHDR_MSGCONTROLLEN) && defined(HAVE_STRUCT_MSGHDR_MSGFLAGS)
+   Debug10("recvmsg(%d, %p{%p,%u,%p,%u,%p,%u,%d}, %d)", s, msgh,
+	  msgh->msg_name, msgh->msg_namelen,  msgh->msg_iov,  msgh->msg_iovlen,
+	  msgh->msg_control,  msgh->msg_controllen,  msgh->msg_flags, flags);
+#else
+   Debug7("recvmsg(%d, %p{%p,%u,%p,%u}, %d)", s, msgh,
+	  msgh->msg_name, msgh->msg_namelen,  msgh->msg_iov,  msgh->msg_iovlen,
+	  flags);
+#endif
    retval = recvmsg(s, msgh, flags);
    _errno = errno;
-   Debug2("recvmsg(, {%s}, ) -> %d",
-	  msgh->msg_name?sockaddr_info(msgh->msg_name, infobuff, sizeof(infobuff)):"NULL",
+#if defined(HAVE_STRUCT_MSGHDR_MSGCONTROLLEN)
+   Debug5("recvmsg(, {%s,%u,,%u,,%u,}, ) -> %d",
+	  msgh->msg_name?sockaddr_info(msgh->msg_name, msgh->msg_namelen, infobuff, sizeof(infobuff)):"NULL",
+	  msgh->msg_namelen, msgh->msg_iovlen, msgh->msg_controllen,
 	  retval);
+#else
+   Debug4("recvmsg(, {%s,%u,,%u,,}, ) -> %d",
+	  msgh->msg_name?sockaddr_info(msgh->msg_name, msgh->msg_namelen, infobuff, sizeof(infobuff)):"NULL",
+	  msgh->msg_namelen, msgh->msg_iovlen,
+	  retval);
+#endif
    errno = _errno;
    return retval;
 }
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
 
-#if WITH_SOCKET
+#if _WITH_SOCKET
 int Send(int s, const void *mesg, size_t len, int flags) {
    int retval, _errno;
    Debug5("send(%d, %p[%08x...], "F_Zu", %d)",
@@ -1051,15 +1103,15 @@ int Send(int s, const void *mesg, size_t len, int flags) {
    errno = _errno;
    return retval;
 }
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
 
-#if WITH_SOCKET
+#if _WITH_SOCKET
 int Sendto(int s, const void *mesg, size_t len, int flags,
 	   const struct sockaddr *to, socklen_t tolen) {
    int retval, _errno;
    char infobuff[256];
 
-   sockaddr_info(to, infobuff, sizeof(infobuff));
+   sockaddr_info(to, tolen, infobuff, sizeof(infobuff));
    Debug7("sendto(%d, %p[%08x...], "F_Zu", %d, {%s}, %d)",
 	  s, mesg, htonl(*(unsigned long *)mesg), len, flags, infobuff, tolen);
    retval = sendto(s, mesg, len, flags, to, tolen);
@@ -1068,9 +1120,9 @@ int Sendto(int s, const void *mesg, size_t len, int flags,
    errno = _errno;
    return retval;
 }
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
 
-#if WITH_SOCKET
+#if _WITH_SOCKET
 int Shutdown(int fd, int how) {
    int retval, _errno;
    Info2("shutdown(%d, %d)", fd, how);
@@ -1080,7 +1132,7 @@ int Shutdown(int fd, int how) {
    errno = _errno;
    return retval;
 }
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
 
 unsigned int Sleep(unsigned int seconds) {
    unsigned int retval;
@@ -1142,7 +1194,7 @@ struct hostent *Gethostbyname(const char *name) {
 }
 #endif /* WITH_IP4 || WITH_IP6 */
 
-#if (WITH_IP4 || WITH_IP6) && HAVE_GETADDRINFO
+#if (_WITH_IP4 || _WITH_IP6) && HAVE_GETADDRINFO
 int Getaddrinfo(const char *node, const char *service,
 		const struct addrinfo *hints, struct addrinfo **res) {
    int result;
@@ -1155,7 +1207,7 @@ int Getaddrinfo(const char *node, const char *service,
    result = getaddrinfo(node, service, hints, res);
    if (result == 0) {
       char sockbuff[256];
-      sockaddr_info((*res)->ai_addr, sockbuff, sizeof(sockbuff));
+      sockaddr_info((*res)->ai_addr, hints->ai_addrlen, sockbuff, sizeof(sockbuff));
       Debug2("getaddrinfo(,,,{{%s, %s}) -> 0",
 	     sockbuff,
 	    (*res)->ai_canonname?(*res)->ai_canonname:"");
@@ -1164,7 +1216,7 @@ int Getaddrinfo(const char *node, const char *service,
    }
    return result;
 }
-#endif /* (WITH_IP4 || WITH_IP6) && HAVE_GETADDRINFO */
+#endif /* (_WITH_IP4 || _WITH_IP6) && HAVE_GETADDRINFO */
 
 #if (WITH_IP4 || WITH_IP6) && HAVE_GETIPNODEBYNAME
 struct hostent *Getipnodebyname(const char *name, int af, int flags,
@@ -1219,6 +1271,7 @@ void *Realloc(void *ptr, size_t size) {
    return result;
 }
 
+#if _WITH_TERMIOS
 int Tcgetattr(int fd, struct termios *termios_p) {
    int i, result, _errno;
    char chars[5*NCCS], *cp = chars;
@@ -1238,7 +1291,9 @@ int Tcgetattr(int fd, struct termios *termios_p) {
    errno = _errno;
    return result;
 }
+#endif /* _WITH_TERMIOS */
 
+#if _WITH_TERMIOS
 int Tcsetattr(int fd, int optional_actions, struct termios *termios_p) {
    int i, result, _errno;
    char chars[5*NCCS], *cp = chars;
@@ -1256,6 +1311,7 @@ int Tcsetattr(int fd, int optional_actions, struct termios *termios_p) {
    errno = _errno;
    return result;
 }
+#endif /* _WITH_TERMIOS */
 
 char *Ttyname(int fd) {
    char *result;
@@ -1393,6 +1449,30 @@ int Mkstemp(char *template) {
    return result;
 }
 
+int Setenv(const char *name, const char *value, int overwrite) {
+   int result, _errno;
+   Debug3("setenv(\"%s\", \"%s\", %d)", name, value, overwrite);
+   result = setenv(name, value, overwrite);
+   _errno = errno;
+   Debug1("setenv() -> %d", result);
+   errno = _errno;
+   return result;
+}
+
+#if HAVE_UNSETENV
+/* on Linux it returns int but on FreeBSD void.
+   we do not expect many errors, so we take void which works on all systems. */
+void Unsetenv(const char *name) {
+   int _errno;
+   Debug1("unsetenv(\"%s\")", name);
+   unsetenv(name);
+   _errno = errno;
+   Debug("unsetenv() ->");
+   errno = _errno;
+   return;
+}
+#endif
+
 #if WITH_READLINE
 
 char *Readline(const char *prompt) {
