@@ -1,5 +1,5 @@
-/* $Id: xio-listen.h,v 1.9 2005/07/30 15:50:12 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2005 */
+/* source: xio-listen.h */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_listen_h_included
@@ -8,7 +8,6 @@
 extern const struct optdesc opt_backlog;
 extern const struct optdesc opt_fork;
 extern const struct optdesc opt_range;
-extern const struct optdesc opt_tcpwrappers;
 
 int
    xioopen_listen(struct single *xfd, int xioflags,
