@@ -1,5 +1,5 @@
-/* $Id: xio-progcall.c,v 1.54 2007/03/06 21:11:55 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2007 */
+/* source: xio-progcall.c */
+/* Copyright Gerhard Rieger 2001-2009 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains common code dealing with program calls (exec, system) */
@@ -10,6 +10,9 @@
 #include "xio-process.h"
 #include "xio-progcall.h"
 
+#include "xio-socket.h"
+
+
 /* these options are used by address pty too */
 #if HAVE_OPENPTY
 const struct optdesc opt_openpty = { "openpty",   NULL, OPT_OPENPTY,     GROUP_PTY,   PH_BIGEN, TYPE_BOOL, 	OFUNC_SPEC };
@@ -44,11 +47,12 @@ const struct optdesc opt_sigquit = { "sigquit",   NULL, OPT_SIGQUIT,     GROUP_P
 int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
 		struct single *fd,
 		unsigned groups,
-		   struct opt **copts	/* in: opts; out: opts for child */
+		   struct opt **copts,	/* in: opts; out: opts for child */
+		   int *duptostderr	/* out: redirect stderr to output fd */
 		) {
    struct opt *popts;	/* parent process options */
    int numleft;
-   int d, type, protocol, sv[2], rdpip[2], wrpip[2];
+   int d, sv[2], rdpip[2], wrpip[2];
    int rw = (xioflags & XIO_ACCMODE);
    bool usepipes = false;
 #if HAVE_PTY
@@ -261,7 +265,7 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
 	    }
 	    if (tn == NULL) {
 	       Error("could not open pty");
-	       return STAT_NORETRY;
+	       return -1;
 	    }
 #endif
 	    Info1("opened pseudo terminal %s", tn);
@@ -280,10 +284,11 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
 #endif /* HAVE_OPENPTY */
       free(*copts);
       if ((*copts = moveopts(popts, GROUP_TERMIOS|GROUP_FORK|GROUP_EXEC|GROUP_PROCESS)) == NULL) {
-	 return STAT_RETRYLATER;
+	 return -1;
       }
       applyopts_cloexec(ptyfd, popts);/*!*/
-      if (fd->howtoend = END_UNSPEC) {
+      /* exec:...,pty did not kill child process under some circumstances */
+      if (fd->howtoend == END_UNSPEC) {
 	 fd->howtoend = END_CLOSE_KILL;
       }
 
@@ -306,7 +311,7 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
       if (rw != XIO_WRONLY) {
 	 if (Pipe(rdpip) < 0) {
 	    Error2("pipe(%p): %s", rdpip, strerror(errno));
-	    return STAT_RETRYLATER;
+	    return -1;
 	 }
       }
       /*0 Info2("pipe({%d,%d})", rdpip[0], rdpip[1]);*/
@@ -314,7 +319,7 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
       free(*copts);
       if ((*copts = moveopts(popts, GROUP_FORK|GROUP_EXEC|GROUP_PROCESS))
 	  == NULL) {
-	 return STAT_RETRYLATER;
+	 return -1;
       }
 
       popts2 = copyopts(popts, GROUP_ALL);
@@ -329,7 +334,7 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
       if (rw != XIO_RDONLY) {
 	 if (Pipe(wrpip) < 0) {
 	    Error2("pipe(%p): %s", wrpip, strerror(errno));
-	    return STAT_RETRYLATER;
+	    return -1;
 	 }
       }
       /*0 Info2("pipe({%d,%d})", wrpip[0], wrpip[1]);*/
@@ -356,20 +361,17 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
       applyopts(fd->fd, popts, PH_LATE);
       if (applyopts_single(fd, popts, PH_LATE) < 0)  return -1;
    } else {
-      d = AF_UNIX;  type = SOCK_STREAM;
-      protocol = 0;	/* PF_UNIX does not work on AIX */
-      retropt_int(popts, OPT_SO_TYPE, &type);
-      result = Socketpair(d, type, protocol, sv);
+      d = AF_UNIX;
+      retropt_int(popts, OPT_PROTOCOL_FAMILY, &d);
+      result = xiosocketpair(popts, d, SOCK_STREAM, 0, sv);
       if (result < 0) {
-	 Error5("socketpair(%d, %d, %d, %p): %s",
-		d, type, protocol, sv, strerror(errno));
-	 return STAT_RETRYLATER;
+	 return -1;
       }
       /*0 Info5("socketpair(%d, %d, %d, {%d,%d})",
 	d, type, protocol, sv[0], sv[1]);*/
       free(*copts);
       if ((*copts = moveopts(popts, GROUP_FORK|GROUP_EXEC|GROUP_PROCESS)) == NULL) {
-	 return STAT_RETRYLATER;
+	 return -1;
       }
       applyopts(sv[0], *copts, PH_PASTSOCKET);
       applyopts(sv[1], popts, PH_PASTSOCKET);
@@ -396,32 +398,15 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
       if (applyopts_single(fd, popts, PH_LATE) < 0)  return -1;
    }
    /*0   if ((optpr = copyopts(*copts, GROUP_PROCESS)) == NULL)
-     return STAT_RETRYLATER;*/
+     return -1;*/
    retropt_bool(*copts, OPT_STDERR, &withstderr);
-   if (Signal(SIGCHLD, childdied) == SIG_ERR) {
-      Warn2("signal(SIGCHLD, %p): %s", childdied, strerror(errno));
-   }
+
+   xiosetchilddied();	/* set SIGCHLD handler */
 
    if (withfork) {
-      const char *forkwaitstring;
-      int forkwaitsecs = 0;
-
-      pid = Fork();
+      pid = xio_fork(true, E_ERROR);
       if (pid < 0) {
-	 Error1("fork(): %s", strerror(errno));
-	 return STAT_RETRYLATER;
-      }
-      /* gdb recommends to have env controlled sleep after fork */
-      if (forkwaitstring = getenv("SOCAT_FORK_WAIT")) {
-         forkwaitsecs = atoi(forkwaitstring);
-         Sleep(forkwaitsecs);
-      }
-
-      if (pid == 0) {	/* child */
-	 /* drop parents locks, reset FIPS... */
-	 if (xio_forked_inchild() != 0) {
-	    Exit(1);
-	 }
+	 return -1;
       }
    }
    if (!withfork || pid == 0) {	/* child */
@@ -435,16 +420,17 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
 
 #if HAVE_PTY
 	 if (usepty) {
+	    Close(ptyfd);
 	    if (rw != XIO_RDONLY && fdi != ttyfd) {
 	       if (Dup2(ttyfd, fdi) < 0) {
 		  Error3("dup2(%d, %d): %s", ttyfd, fdi, strerror(errno));
-		  return STAT_RETRYLATER; }
+		  return -1; }
 	       /*0 Info2("dup2(%d, %d)", ttyfd, fdi);*/
 	    }
 	    if (rw != XIO_WRONLY && fdo != ttyfd) {
 	       if (Dup2(ttyfd, fdo) < 0) {
 		  Error3("dup2(%d, %d): %s", ttyfd, fdo, strerror(errno));
-		  return STAT_RETRYLATER; }
+		  return -1; }
 	       /*0 Info2("dup2(%d, %d)", ttyfd, fdo);*/
 	    }
 	    if ((rw == XIO_RDONLY || fdi != ttyfd) &&
@@ -463,10 +449,12 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
 		  a graceful solution via temporary descriptors */
 	       int tmpi, tmpo;
 
+	       if (rw != XIO_WRONLY)  Close(rdpip[0]);
+	       if (rw != XIO_RDONLY)  Close(wrpip[1]);
 	       if (fdi == rdpip[1]) {	/* a conflict here */
 		  if ((tmpi = Dup(wrpip[0])) < 0) {
 		     Error2("dup(%d): %s", wrpip[0], strerror(errno));
-		     return STAT_RETRYLATER;
+		     return -1;
 		  }
 		  /*0 Info2("dup(%d) -> %d", wrpip[0], tmpi);*/
 		  rdpip[1] = tmpi;
@@ -474,7 +462,7 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
 	       if (fdo == wrpip[0]) {	/* a conflict here */
 		  if ((tmpo = Dup(rdpip[1])) < 0) {
 		     Error2("dup(%d): %s", rdpip[1], strerror(errno));
-		     return STAT_RETRYLATER;
+		     return -1;
 		  }
 		  /*0 Info2("dup(%d) -> %d", rdpip[1], tmpo);*/
 		  wrpip[0] = tmpo;
@@ -483,7 +471,7 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
 	       if (rw != XIO_WRONLY && rdpip[1] != fdo) {
 		  if (Dup2(rdpip[1], fdo) < 0) {
 		     Error3("dup2(%d, %d): %s", rdpip[1], fdo, strerror(errno));
-		     return STAT_RETRYLATER;
+		     return -1;
 		  }
 		  Close(rdpip[1]);
 		  /*0 Info2("dup2(%d, %d)", rdpip[1], fdo);*/
@@ -492,7 +480,7 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
 	       if (rw != XIO_RDONLY && wrpip[0] != fdi) {
 		  if (Dup2(wrpip[0], fdi) < 0) {
 		     Error3("dup2(%d, %d): %s", wrpip[0], fdi, strerror(errno));
-		     return STAT_RETRYLATER;
+		     return -1;
 		  }
 		  Close(wrpip[0]);
 		  /*0 Info2("dup2(%d, %d)", wrpip[0], fdi);*/
@@ -506,20 +494,22 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
 	       applyopts(fdo, *copts, PH_LATE2);
 
 	    } else {	/* socketpair */
+	       Close(sv[0]);
 	       if (rw != XIO_RDONLY && fdi != sv[1]) {
 		  if (Dup2(sv[1], fdi) < 0) {
 		     Error3("dup2(%d, %d): %s", sv[1], fdi, strerror(errno));
-		     return STAT_RETRYLATER; }
+		     return -1; }
 		  /*0 Info2("dup2(%d, %d)", sv[1], fdi);*/
 	       }
 	       if (rw != XIO_WRONLY && fdo != sv[1]) {
 		  if (Dup2(sv[1], fdo) < 0) {
 		     Error3("dup2(%d, %d): %s", sv[1], fdo, strerror(errno));
-		     return STAT_RETRYLATER; }
+		     return -1; }
 		  /*0 Info2("dup2(%d, %d)", sv[1], fdo);*/
 	       }
 	       if (fdi != sv[1] && fdo != sv[1]) {
 		  applyopts_cloexec(sv[1], *copts);
+		  Close(sv[1]);
 	       }
 
 	       applyopts(fdi, *copts, PH_LATE);
@@ -530,36 +520,6 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
 	 applyopts(-1, *copts, PH_LATE);
 	 applyopts(-1, *copts, PH_LATE2);
       }
-
-      /* what to do with stderr? */
-      if (withstderr) {
-	 /* handle it just like ordinary process output, i.e. copy output fd */
-	 if (!withfork) {
-	    if (Dup2(fdo, 2) < 0) {
-	       Error2("dup2(%d, 2): %s", fdo, strerror(errno));
-	    }
-	    /*0 Info1("dup2(%d, 2)", fdo);*/
-	 } else
-#if HAVE_PTY
-	 if (usepty) {
-	    if (Dup2(ttyfd, 2) < 0) {
-	       Error2("dup2(%d, 2): %s", ttyfd, strerror(errno));
-	    }
-	    /*0 Info1("dup2(%d, 2)", ttyfd);*/
-	 } else
-#endif /* HAVE_PTY */
-	 if (usepipes) {
-	    if (Dup2(/*rdpip[1]*/ fdo, 2) < 0) {
-	       Error2("dup2(%d, 2): %s", /*rdpip[1]*/ fdo, strerror(errno));
-	    }
-	    /*0 Info1("dup2(%d, 2)", rdpip[1]);*/
-	 } else {
-	    if (Dup2(sv[1], 2) < 0) {
-	       Error2("dup2(%d, 2): %s", sv[1], strerror(errno));
-	    }
-	    /*0 Info1("dup2(%d, 2)", sv[1]);*/
-	 }
-      }
       _xioopen_setdelayeduser();
       /* set group before user - maybe you are not permitted afterwards */
       if (retropt_gidt(*copts, OPT_SETGID, &group) >= 0) {
@@ -568,6 +528,12 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
       if (retropt_uidt(*copts, OPT_SETUID, &user) >= 0) {
 	 Setuid(user);
       }
+      if (withstderr) {
+	 *duptostderr = fdo;
+      } else {
+	 *duptostderr = -1;
+      }
+
       return 0;	/* indicate child process */
    }
 
@@ -585,13 +551,14 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
       if (Close(ttyfd) < 0) {
 	 Info2("close(%d): %s", ttyfd, strerror(errno));
       }
-   } /*0 else*/
+   } else
 #endif /* HAVE_PTY */
-#if 0
    if (usepipes) {
+      Close(rdpip[1]);
+      Close(wrpip[0]);
    } else {
+      Close(sv[1]);
    }
-#endif
    fd->para.exec.pid = pid;
 
    if (applyopts_single(fd, popts, PH_LATE) < 0)  return -1;
