@@ -1,5 +1,5 @@
-/* $Id: xio.h,v 1.66 2007/03/06 21:25:51 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2007 */
+/* source: xio.h */
+/* Copyright Gerhard Rieger 2001-2009 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_h_included
@@ -146,6 +146,13 @@ typedef struct single {
    bool   opt_unlink_close;	/* option unlink_close */
    char  *unlink_close;	/* name of a symlink or unix socket to be removed */
    int dtype;
+   enum {
+      XIOSHUT_UNSPEC,	/* standard (address dependent) behaviour */
+      XIOSHUT_NONE,	/* do nothing on shutdown */
+      XIOSHUT_CLOSE,	/* close the FD */
+      XIOSHUT_DOWN,	/* shutdown() */
+      XIOSHUT_NULL	/* send an empty packet (dgram socket) */
+   } howtoshut;
    enum {
       END_UNSPEC,	/* after init, when no end-close... option */
       END_NONE,		/* no action */
@@ -167,6 +174,8 @@ typedef struct single {
    const char *name;		/* only with END_UNLINK */
    int (*sigchild)(struct single *);	/* callback after sigchild */
    pid_t ppid;			/* parent pid, only if we send it signals */
+   int escape;			/* escape character; -1 for no escape */
+   bool actescape;		/* escape character found in input data */
    union {
       struct {
 	 int fdout;		/* use fd for output */
@@ -175,9 +184,9 @@ typedef struct single {
       struct {
 	 struct timeval connect_timeout; /* how long to hang in connect() */
 	 union sockaddr_union la;	/* local socket address */
-	 bool emptyiseof;	/* with dgram: empty packet means EOF */
+	 bool null_eof;		/* with dgram: empty packet means EOF */
 	 bool dorange;
-	 union xiorange_union range;	/* restrictions for peer address */
+	 struct xiorange range;	/* restrictions for peer address */
 #if _WITH_IP4 || _WITH_IP6
 	 struct {
 	    unsigned int res_opts[2];	/* bits to be set in _res.options are
@@ -333,7 +342,7 @@ union integral {
    } u_ip_mreq;
 #endif
 #if WITH_IP4
-   in_addr_t  u_ip4addr;
+   struct in_addr  u_ip4addr;
 #endif
 } ;
 
@@ -355,6 +364,8 @@ union integral {
 struct opt {
    const struct optdesc *desc;
    union integral value;
+   union integral value2;
+   union integral value3;
 } ;
 
 extern const char *PIPESEP;
@@ -373,6 +384,8 @@ extern xiofile_t *sock[XIO_MAXSOCK];
 				   not even by external changes correctable */
 
 extern int xioinitialize(void);
+extern int xioinitialize2(void);
+extern pid_t xio_fork(bool subchild, int level);
 extern int xio_forked_inchild(void);
 extern int xiosetopt(char what, const char *arg);
 extern int xioinqopt(char what, char *arg, size_t n);
@@ -388,6 +401,7 @@ extern pid_t diedunknown3;
 extern pid_t diedunknown4;
 
 extern int xiosetsigchild(xiofile_t *xfd, int (*callback)(struct single *));
+extern int xiosetchilddied(void);
 extern int xio_opt_signal(pid_t pid, int signum);
 extern void childdied(int signum);
 
